@@ -36,3 +36,6 @@
 
 /* Define if you have a new bzip2 implementation that uses BZ2_name */
 #undef HAVE_NEW_BZDOPEN
+
+/* Define if you have /dev/urandom */
+#undef HAVE_DEV_URANDOM
