@@ -25,10 +25,10 @@
 #include "dact.h"
 #include "buffer.h"
 
-TWO_BYTES bit_buffer_data=0;
+uint32_t bit_buffer_data=0;
 int bit_buffer_location=0;
 
-char byte_buffer_data[BYTE_BUFF_SIZE];
+char *byte_buffer_data=NULL;
 int byte_buffer_location=-1;
 
 
@@ -44,7 +44,7 @@ int byte_buffer_location=-1;
 int bit_buffer_read(int bits) {
         int retval;
         if (bits>bit_buffer_location) { 
-		printf("buffer.c: bit_buffer_read: Request for %i bits, buffer contains %i bits\n",bits,bit_buffer_location);
+		fprintf(stderr, __FILE__ ": bit_buffer_read: Request for %i bits, buffer contains %i bits\n",bits,bit_buffer_location);
 		bits=bit_buffer_location;
 	}
 
@@ -52,7 +52,7 @@ int bit_buffer_read(int bits) {
         bit_buffer_data=(bit_buffer_data<<bits);
 	bit_buffer_location-=bits;
 #ifdef BUFFER_VERBOSE
-	printf("buffer.c: bit_buffer_read(%i)=%i\n",bits,retval);
+	fprintf(stderr, __FILE__ ": bit_buffer_read(%i)=%i\n",bits,retval);
 #endif
         return(retval);
 }
@@ -68,18 +68,19 @@ int bit_buffer_read(int bits) {
     Stat: Complete
 */
 void bit_buffer_write(unsigned int val, unsigned int bits) {
-        if ((bits+bit_buffer_location)>(sizeof(bit_buffer_data)*8)) {
-                printf("buffer.c: bit_buffer_write: Buffer overflow\n");
-                return;
+        while ((val>>bits)!=0) {
+                fprintf(stderr, __FILE__ ": bit_buffer_write: Value (%i) is bigger than %i bits.\n",val,bits);
+		bits++;
         }
-        if ((val>>bits)!=0) {
-                printf("buffer.c: bit_buffer_write: Value (%i) is bigger than %i bits.\n",val,bits);
+
+        if ((bits+bit_buffer_location)>(sizeof(bit_buffer_data)*8)) {
+                fprintf(stderr, __FILE__ ": bit_buffer_write: Buffer overflow\n");
                 return;
         }
         bit_buffer_location+=bits;
         bit_buffer_data+=(val<<((sizeof(bit_buffer_data)*8)-bit_buffer_location));
 #ifdef BUFFER_VERBOSE
-	printf("buffer.c: bit_buffer_write(%i, %i)\n",val,bits);
+	fprintf(stderr, __FILE__ ": bit_buffer_write(%i, %i)\n",val,bits);
 #endif
         return;
 }
@@ -94,7 +95,7 @@ void bit_buffer_write(unsigned int val, unsigned int bits) {
 */
 int bit_buffer_size(void) {
 #ifdef BUFFER_VERBOSE
-	printf("buffer.c: bit_buffer_size()=%i\n",bit_buffer_location);
+	fprintf(stderr, __FILE__ ": bit_buffer_size()=%i\n",bit_buffer_location);
 #endif
         return(bit_buffer_location);
 }
@@ -110,7 +111,7 @@ void bit_buffer_purge(void) {
         bit_buffer_location=0;
         bit_buffer_data=0;
 #ifdef BUFFER_VERBOSE
-	printf("buffer.c: bit_buffer_purge()\n");
+	fprintf(stderr, __FILE__ ": bit_buffer_purge()\n");
 #endif
         return;
 }
@@ -120,7 +121,7 @@ char *byte_buffer_read(int bytes) {
 	char *retval;
 	if (byte_buffer_location==-1) return(NULL);
 	if (bytes>byte_buffer_location) {
-		printf("buffer.c: byte_buffer_read: Request for %i bytes, buffer contains %i bytes.\n",bytes,byte_buffer_location);
+		fprintf(stderr, __FILE__ ": byte_buffer_read: Request for %i bytes, buffer contains %i bytes.\n",bytes,byte_buffer_location);
 		bytes=byte_buffer_location;
 	}
 	retval=malloc(bytes);
@@ -130,9 +131,11 @@ char *byte_buffer_read(int bytes) {
 }
 
 void byte_buffer_write(char *val, unsigned int size) {
-	if (byte_buffer_location==-1) byte_buffer_purge();
-	if ((byte_buffer_location+size)>sizeof(byte_buffer_data)) {
-		printf("buffer.c: byte_buffer_write: Buffer overflow.\n");
+	if (byte_buffer_location==-1) {
+		byte_buffer_purge();
+	}
+	if ((byte_buffer_location+size)>(BYTE_BUFF_SIZE)) {
+		fprintf(stderr, __FILE__ ": byte_buffer_write: Buffer overflow.\n");
 		return;
 	}
 	memcpy(byte_buffer_data+byte_buffer_location,val,size);
@@ -154,8 +157,14 @@ int byte_buffer_size(void) {
 
 void byte_buffer_purge(void) {
 	int i;
+
+#if 0
+	if (byte_buffer_data!=NULL) free(byte_buffer_data);
+	byte_buffer_data=calloc(BYTE_BUFF_SIZE, 1);
+#endif
+	if (byte_buffer_data==NULL) byte_buffer_data=malloc(BYTE_BUFF_SIZE);
 	if (byte_buffer_location==-1) {
-		for (i=0;i<BYTE_BUFF_SIZE;i++) byte_buffer_data[i]=0;
+		for (i=0;i<(BYTE_BUFF_SIZE);i++) byte_buffer_data[i]=0;
 	} else {
 		for (i=0;i<(byte_buffer_location+1);i++) byte_buffer_data[i]=0;
 	}
