@@ -1,3 +1,5 @@
+#ifndef _COMP_BZLIB_H
+#define _COMP_BZLIB_H
 #include "dact.h"
 #include "config.h"
 
@@ -8,3 +10,4 @@ int comp_bzlib_algo(int mode, unsigned char *prev_block, unsigned char *curr_blo
 int comp_bzlib_compress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 int comp_bzlib_decompress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 #endif
+#endif
