@@ -1,7 +1,11 @@
 #ifndef _COMP_FACTOR_H
 #define _COMP_FACTOR_H
-/* XXX: Add module support! */
+
+#if defined(USE_MODULES) && !defined(AS_MODULE)
+#define comp_factor_algo DACT_FAILED_ALGO
+#else
 int comp_factor_algo(int mode, unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 int comp_factor_compress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 int comp_factor_decompress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 #endif
+#endif
