@@ -1,3 +1,5 @@
+#ifndef _COMP_RANGE_H
+#define _COMP_RANGE_H
 #include "dact.h"
 #include "config.h"
 
@@ -10,3 +12,4 @@ int comp_range_decompress(unsigned char *prev_block, unsigned char *curr_block,
 #endif
 
 void incoming_data(unsigned char value);
+#endif
