@@ -1,3 +1,5 @@
+#ifndef _COMP_ZLIB_H
+#define _COMP_ZLIB_H
 #include "dact.h"
 #include "config.h"
 
@@ -8,3 +10,4 @@ int comp_zlib_algo(int mode, unsigned char *prev_block, unsigned char *curr_bloc
 int comp_zlib_compress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 int comp_zlib_decompress(unsigned char *prev_block, unsigned char *curr_block, char *out_block, int blk_size);
 #endif
+#endif
