@@ -9,10 +9,11 @@
 #include "dact.h"
 #include "parse.h"
 #include "dendian.h"
+#include "crc.h"
 #include "math.h"
 #include "algorithms.h"
+#include "ciphers.h"
 #include "module.h"
-#include "crc.h"
 #include "header.h"
 #include "parse.h"
 #include "net.h"
@@ -24,19 +25,25 @@
 #include <bzlib.h>
 #endif
 
+
+int print_help(int argc, char **argv);
+void dact_loadconfig(const char *path, char *options);
+int dact_upgrade(const char *options, uint32_t *crcs);
+uint32_t is_latest (const char *options);
+int dact_shutdown(int retval);
+char *dact_getoutfilename(const char *orig, const int mode);
+uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const char *options);
+uint32_t dact_blk_decompress(char *ret, const char *srcbuf, const uint32_t size, const char *options, const int algo);
+uint32_t dact_blk_compress(char *algo, char *ret, const char *srcbuf, const uint32_t size, const char *options);
+uint32_t dact_process_file(const int src, const int dest, const int mode, const char *options, const char *filename, uint32_t *crcs, int cipher);
+int main(int argc, char **argv);
+
 extern char *optarg;
 extern int optind, opterr, optopt;
 uint32_t DACT_BLK_SIZE=4088;
 
 int print_help(int argc, char **argv) {
-	printf("DACT %i.%i.%i-%s by Netfuel Design. <dact@rkeene.org>\n",
-		DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION,
-#ifdef DEBUG
-		"dev"
-#else
-		"rel"
-#endif
-		);
+	printf("DACT %i.%i.%i-%s by Netfuel Design. <dact@rkeene.org>\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
 	printf("usage: %s [options...] [file ...]\n",argv[0]);
 
 	printf("Options:\n");
@@ -54,13 +61,17 @@ int print_help(int argc, char **argv) {
 	printf("  -O          Toggle writing original file name.\n");
 	printf("  -S          Use speed-size as a metric rather than size.\n");
 	printf("  -h          Give this help.\n");
+	printf("  -V          Display DACT version (%i.%i.%i-%s).\n", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
+	printf("  -N          Upgrade DACT.\n");
 	printf("  -b NN       Use a block size of NN bytes for compression.\n");
 	printf("  -e NN       Exclude algorithm NN from being used.\n");
 	printf("  -m CONF     Load config file CONF.\n");
 	printf("  -u URL      Specify download location as URL.\n");
+	printf("  -p URL      Parse URL and print results, then exit.\n");
 	printf("  -D DESC     Specy a description of DESC.\n");
 	printf("  -I NN       Use ONLY 2 algorithms, NN and 0.\n");
 	printf("  -U FILE     Use FILE to select download location.\n");
+	printf("  -E CIPHER   Use CIPHER to encrypt data (LIST  lists available ciphers.)\n");
 	printf("  file...     File(s) to (de)compress.  (If none given, use standard input).\n");
 	return(0);
 }
@@ -106,6 +117,9 @@ void dact_loadconfig(const char *path, char *options) {
 				}
 #endif
 				break;
+			case 168825941: /* block_size */
+				DACT_BLK_SIZE=atoi2(item_buf[1]);
+				break;
 			case 162975987: /* use_urls */
 				options[DACT_OPT_URL]=!!strcmp(item_buf[1],"off");
 				break;
@@ -125,6 +139,57 @@ void dact_loadconfig(const char *path, char *options) {
 }
 
 
+int dact_upgrade(const char *options, uint32_t *crcs) {
+	char *urlsubst, *buf;
+	char dact_binfile[256];
+	int inFd, outFd;
+	uint32_t i;
+
+	if ((i=is_latest(options))) {
+		PRINTERR("**+");
+		PRINT_LINE; fprintf(stderr, "dact: **> CURR: DACT %i.%i.%i\n",DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION);
+		PRINT_LINE; fprintf(stderr, "dact: **> NEW:  DACT %i.%i.%i\n",i>>16,(i>>8)&0xff,i&0xff);
+		PRINTERR("**>");
+		PRINTERR("**-");
+	}
+	if ((inFd=open_net("http://www.rkeene.org/projects/rget/rget.cgi?project=dact&file=info", O_RDONLY))>=0) {
+		fprintf(stderr, "------------------------\n");
+		buf=malloc(1024);
+		while (1) {
+			i=read_f(inFd, buf, 1024);
+			write(STDERR_FILENO, buf, i);
+			if (i!=1024) break;
+		}
+		fprintf(stderr, "------------------------\n");
+		close(inFd);
+	}
+
+#ifdef GO32
+	mkdir("c:/dact/", 0755);
+	strcpy(dact_binfile,"c:/dact/dact.exe");
+#else
+	strncpy(dact_binfile,getenv("HOME"),sizeof(dact_binfile)-1);
+	strncat(dact_binfile,"/.dact/",sizeof(dact_binfile)-strlen(dact_binfile)-1);
+	mkdir(dact_binfile, 0755);
+	strncat(dact_binfile,"dact.bin",sizeof(dact_binfile)-strlen(dact_binfile)-1);
+#endif
+	urlsubst=parse_url_subst("http://www.rkeene.org/projects/rget/rget.cgi?os=@@OSNM@@&arch=@@ARCH@@&project=dact&file=bin&meth=gz","");
+	if ((outFd=open_net(dact_binfile, O_WRONLY|O_TRUNC|O_CREAT, 0755))<0) { PERROR("open_net"); return(-1); }
+	if ((inFd=open_net(urlsubst, O_RDONLY))<0) { PERROR("open_net"); return(-1); }
+	if (!dact_process_file(inFd, outFd, DACT_MODE_DECMP, options, "dact", crcs,-1)) {
+		close(inFd);
+		close(outFd);
+		unlink(dact_binfile);
+		PRINTERR("Failed to upgrade DACT.");
+		return(-1);
+	}
+	close(inFd);
+	close(outFd);
+	if (!options[DACT_OPT_BINCHK]) {
+		PRINTERR("Note: You do not have binary_check  set to `on' in your dact.conf.");
+	}
+	return(1);
+}
 
 uint32_t is_latest (const char *options) {
 #if defined(CHECK_VERSION)
@@ -220,6 +285,7 @@ uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const
  *  -- Roy Keene <rkeene@rkeene.org>
  */
 	if (lseek_net(src, 0, SEEK_SET)<0) {
+		PRINTERR("This should not be happening.");
 /*
  * lseek_net() should make this obsolete.
  *
@@ -232,6 +298,7 @@ uint32_t dact_process_other(int src, const int dest, const uint32_t magic, const
 			write(tmpfd, buf, x);
 			if (x<1024) break;
 		}
+		close(src);
 		src=tmpfd;
 		lseek_net(src, 0, SEEK_SET); /* Now bitch. */
 		free(buf);
@@ -370,13 +437,13 @@ uint32_t dact_blk_compress(char *algo, char *ret, const char *srcbuf, const uint
 	return(smallest_size);
 }
 
-uint32_t dact_process_file(const int src, const int dest, const int mode, const char *options, const char *filename, uint32_t *crcs) {
+uint32_t dact_process_file(const int src, const int dest, const int mode, const char *options, const char *filename, uint32_t *crcs, int cipher) {
 	struct stat filestats;
 	FILE *extd_urlfile;
 	char *file_extd_urls[256];
 	unsigned char algo;
 	char ch;
-	char *in_buf, *out_buf, *hdr_buf;
+	char *in_buf, *out_buf, *hdr_buf, *keybuf=NULL, *tmpbuf=NULL;
 	char version[3]={DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION};
 	char file_opts=0;
 	uint32_t bytes_read, retsize;
@@ -395,7 +462,13 @@ uint32_t dact_process_file(const int src, const int dest, const int mode, const
 		}
 
 		dact_ui_setup(((float) (filestats.st_size/DACT_BLK_SIZE)+0.9999));
+		if (cipher!=-1) {
+			dact_hdr_ext_regn(DACT_HDR_CIPHER, cipher, sizeof(cipher));
+			keybuf=malloc(DACT_KEY_SIZE);
 
+			ciphers[cipher](NULL, NULL, 0, keybuf, DACT_MODE_CINIT+DACT_MODE_CENC);
+			
+		}
 		blksize_size=BYTESIZE(DACT_BLK_SIZE);
 
 		if (!options[DACT_OPT_ORIG] && filename!=NULL)
@@ -425,6 +498,15 @@ uint32_t dact_process_file(const int src, const int dest, const int mode, const
 			blk_cnt++;
 
 			retsize=dact_blk_compress(&algo, out_buf, in_buf, DACT_BLK_SIZE, options);
+
+/* CIPHER the data if an encryption algorithm is specified. */
+			if (cipher!=-1) {
+				tmpbuf=malloc(retsize*2);
+				x=ciphers[cipher](out_buf, tmpbuf, retsize, keybuf, DACT_MODE_CENC);
+				memcpy(out_buf,tmpbuf,x);
+				free(tmpbuf);
+			}
+
 			if (retsize>0) {
 				if (options[DACT_OPT_VERB]>1) {
 					if (options[DACT_OPT_VERB]>2) {
@@ -574,6 +656,9 @@ XXX: Todo, make this do something...
 					fprintf(stderr, "DESC: %s\n",hdr_buf);
 					free(hdr_buf);
 					break;
+				case DACT_HDR_CIPHER:
+					read_de(src,&cipher,x);
+					break;
 				case DACT_HDR_NOP:
 					x=-2;
 					break;
@@ -600,7 +685,7 @@ XXX: Todo, make this do something...
 				close(src);
 				crcs[4]=crcs[2];
 				crcs[5]=crcs[3];
-				return(dact_process_file(new_fd, dest, mode, options, filename, crcs));
+				return(dact_process_file(new_fd, dest, mode, options, filename, crcs, cipher));
 			}
 		}
 
@@ -615,7 +700,7 @@ XXX: Todo, make this do something...
 */
 		if (filesize==0) {
 /* See if we can rewind our stream, so when we get to the end, we can come back! */
-			if (lseek_net(src, 0, SEEK_SET)==0) {
+			if (lseek_net(src, 1, SEEK_SET)==0) {
 				canlseek=1;
 				lseek_net(src, -8, SEEK_END);
 				read_de(src, &magic, 4);
@@ -641,6 +726,14 @@ XXX: Todo, make this do something...
 
 		dact_ui_setup((int)(((float) filesize/(float) DACT_BLK_SIZE)+0.9999));
 
+		if (cipher!=-1) {
+			keybuf=malloc(DACT_KEY_SIZE);
+			ciphers[cipher](NULL, NULL, 0, keybuf, DACT_MODE_CINIT+DACT_MODE_CDEC);
+			
+		}
+
+
+
 		while (1) {
 			if (read(src, &algo, 1)==0) break;
 			if (algo==0xff) break; /* 0xff is reserved for EOF */
@@ -655,6 +748,15 @@ XXX: Todo, make this do something...
 
 			read_f(src, in_buf, blksize);
 
+			crcs[0]=ELFCRC(crcs[0],in_buf,blksize);
+
+			if (cipher!=-1) {
+				tmpbuf=malloc(blksize*2);
+				x=ciphers[cipher](in_buf, tmpbuf, blksize, keybuf, DACT_MODE_CDEC);
+				memcpy(in_buf,tmpbuf,x);
+				free(tmpbuf);
+			}
+
 /*
  * If the filesize is not specified, try to find it in the stream...
  * this is pretty stupid, because if we can't rewind OUR read stream
@@ -684,14 +786,16 @@ XXX: Todo, make this do something...
 
 
 			if ((bytes_read=dact_blk_decompress(out_buf, in_buf, blksize, 0, algo))==0) {
-				PRINTERR("Decompression resulted in 0-byte block.");
+				if (cipher!=-1) {
+					PRINTERR("Decompression resulted in 0-byte block.  Invalid key?");
+				} else {
+					PRINTERR("Decompression resulted in 0-byte block.");
+				}
 			}
 			fileoutsize+=bytes_read;
 
-			crcs[0]=ELFCRC(crcs[0],in_buf,blksize);
 			crcs[1]=ELFCRC(crcs[1],out_buf,bytes_read);
 
-
 			dact_ui_incrblkcnt(1);
 
 			if (fileoutsize>filesize && filesize!=0) {
@@ -812,6 +916,10 @@ XXX: Todo, make this do something...
 					printf("Description       :   %s\n", hdr_buf);
 					free(hdr_buf);
 					break;
+				case DACT_HDR_CIPHER:
+					read_de(src, &cipher, x);
+					printf("Ciphered using    :   %s\n", ciphers_name[cipher]);
+					break;
 				case DACT_HDR_NOP:
 					x=-2;
 					break;
@@ -854,7 +962,7 @@ int main(int argc, char **argv) {
 	char dact_binfile[256];
 	int filecnt=0;
 	int in_fd, out_fd;
-	int mode=DACT_MODE_COMPR;
+	int mode=DACT_MODE_COMPR, ciphernum=-1;
 	uint32_t crcs[6]={0,0,0,0,0,0};
 	uint32_t i,x;
 
@@ -867,6 +975,12 @@ int main(int argc, char **argv) {
 	strncat(dact_binfile,"/.dact/dact.conf",sizeof(dact_binfile)-strlen(dact_binfile)-1);
 	dact_loadconfig(dact_binfile, options);
 #endif
+/* XXX: hack, to make upgrade work even if DACT_OPT_BINCHK is enabled, we must
+ *      get the new version before executing it.
+ */
+	if (argv[1]!=NULL) {
+		if (!strcmp(argv[1],"-N")) { return(dact_upgrade(options,crcs)); }
+	}
 
 	if (options[DACT_OPT_BINCHK]) {
 #ifdef GO32
@@ -884,8 +998,11 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	while ((opt=getopt(argc,argv,"dfsvcnHCp:I:m:e:lb:hu:U:TPOD:"))!=-1) {
+	while ((opt=getopt(argc,argv,"adfsvcnNVHCE:p:I:m:e:lb:hu:U:TPOD:"))!=-1) {
 		switch (opt) {
+			case 'a':
+				PRINTERR("This option does nothing.");
+				break;
 			case 'd':
 				mode=DACT_MODE_DECMP;
 				break;
@@ -926,6 +1043,30 @@ int main(int argc, char **argv) {
 			case 'H':
 				options[DACT_OPT_HDONLY]=!options[DACT_OPT_HDONLY];
 				break;
+			case 'N':
+				return(dact_upgrade(options,crcs));
+				break;
+			case 'E':
+				strtolower(optarg);
+				x=hash_fourbyte(optarg, ' ');
+				if (x==hash_fourbyte("list", ' ')) {
+					for (i=0;i<CIPHER_COUNT;i++) {
+						if (ciphers_name[i]!=NULL) {
+							fprintf(stderr, "%i -  %s\n",i,ciphers_name[i]);
+						}
+					}
+					return(1);
+				}
+				for (i=0;i<CIPHER_COUNT;i++) {
+					if (ciphers_name[i]!=NULL) {
+						if (x==hash_fourbyte(ciphers_name[i], ' ')) {
+							break;
+						}
+					}
+				}
+				if (i==CIPHER_COUNT) i=-1;
+				ciphernum=i;
+				break;
 			case 'l':
 				for (i=0;i<255;i++) {
 					if (algorithms[i]==NULL || algorithms[i]==DACT_FAILED_ALGO) continue;
@@ -940,6 +1081,17 @@ int main(int argc, char **argv) {
 					if (i!=x) algorithms[i]=NULL;
 				}
 				break;
+			case 'V':
+				printf("DACT %i.%i.%i-%s", DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION, DACT_VER_SUB);
+#if defined(__DATE__) && defined(__TIME__)
+				printf("  built on %s at %s",__DATE__,__TIME__);
+#endif
+#ifdef DACT_CONTACT
+				printf(" %s",DACT_CONTACT);
+#endif
+				printf("\n");
+				return(0);
+				break;
 			case 'u':
 				dact_hdr_ext_regs(DACT_HDR_URL, optarg, strlen(optarg));
 				break;
@@ -979,7 +1131,7 @@ int main(int argc, char **argv) {
 		PRINT_LINE; fprintf(stderr, "dact: **> [CURR: DACT %i.%i.%i]\n",DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION);
 		PRINT_LINE; fprintf(stderr, "dact: **> [NEW:  DACT %i.%i.%i]\n",i>>16,(i>>8)&0xff,i&0xff);
 		PRINTERR("**>");
-		PRINTERR("**> Run `dact-upgrade.sh' if you installed it to get it.");
+		PRINTERR("**> Run `dact -N' to get it.");
 		PRINTERR("**> or get the source at: http://www.rkeene.org/devel/dact.tar.gz");
 		PRINTERR("**>");
 		PRINTERR("**-");
@@ -1020,9 +1172,7 @@ int main(int argc, char **argv) {
 					close(in_fd);
 					continue;
 				}
-				if (options[DACT_OPT_STDOUT]) {
-					out_fd=STDOUT_FILENO;
-				} else {
+				if (!options[DACT_OPT_STDOUT]) {
 					if ((out_fd=open_net(out_file,O_WRONLY|O_CREAT|O_TRUNC,0644))==-1) {
 						fprintf(stderr, "dact: Can't open %s for writing.\n",out_file);
 						PERROR("open");
@@ -1030,6 +1180,7 @@ int main(int argc, char **argv) {
 					}
 				}
 			}
+			if (options[DACT_OPT_STDOUT]) out_fd=STDOUT_FILENO;
 		}
 
 /* Use STDIN/STDOUT if no files specified ... */
@@ -1046,7 +1197,7 @@ int main(int argc, char **argv) {
 /* Okay, we're all done, now pass these to something to do the real stuff */
 		if (in_fd!=-1 && (out_fd!=-1 || mode==DACT_MODE_STAT)) {
 			crcs[1]=crcs[0]=0;
-			if (dact_process_file(in_fd, out_fd, mode, options, in_file, crcs)==0) {
+			if (dact_process_file(in_fd, out_fd, mode, options, in_file, crcs, ciphernum)==0) {
 				close(in_fd);
 				close(out_fd);
 				if (out_fd!=STDOUT_FILENO) {
