@@ -3,14 +3,30 @@
 
 #include "config.h"
 
+#ifndef DACT_CONTACT
+#define DACT_CONTACT "<dact@rkeene.org>"
+#endif
+
+#ifndef DACT_KEY_SIZE
+#define DACT_KEY_SIZE 2048
+#endif
+
 #define DACT_VER_MAJOR 0
-#define DACT_VER_MINOR 7
-#define DACT_VER_REVISION 1
+#define DACT_VER_MINOR 8
+#define DACT_VER_REVISION 0
+#ifdef DEBUG
+#define DACT_VER_SUB	"dev"
+#else
+#define DACT_VER_SUB	"rel"
+#endif
 
 #define DACT_MODE_COMPR 1
 #define DACT_MODE_DECMP 2
 #define DACT_MODE_STAT  3
 #define DACT_MODE_RET   4
+#define DACT_MODE_CINIT	5
+#define DACT_MODE_CENC	6
+#define DACT_MODE_CDEC	7
 
 #define DACT_OPT_FORCE	0
 #define DACT_OPT_STDOUT	1
