@@ -1,5 +1,5 @@
-#ifndef __LOCAL_MATH_H
-#define __LOCAL_MATH_H
+#ifndef _LOCAL_MATH_H
+#define _LOCAL_MATH_H
 #include <math.h>
 
 #ifndef M_LN2l
