@@ -1,4 +1,7 @@
+#ifndef _DACT_SORT_H
+#define _DACT_SORT_H
 #include "dact.h"
 void int_sort (uint32_t array[], unsigned int listsize, int mode);
 void int_sort_fast (uint32_t array[], unsigned int elements, int mode);
 void int_sort_really_fast (uint32_t array[], unsigned int elements, int mode);
+#endif
