@@ -1,3 +1,7 @@
 /* Compliments of Jay Freeman <saurik@saurik.com> */
 
+#ifndef _DACT_STRSEP_H
+#define _DACT_STRSEP_H
+
 char *strsep(char **stringp, const char *delim);
+#endif
